//------------------------------------------------------------------------------
// GPL LISENCE (short)
//------------------------------------------------------------------------------
/**
 * Copyright (c) 2013 Reiko1231
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//------------------------------------------------------------------------------
// PLUGIN INFORMATION
//------------------------------------------------------------------------------
public Plugin:myinfo = 
{
	name 		= "[ Menu Creator ]",
	author 		= "Reiko1231",
	description = "Simple creating of menu and panels for Source Games",
	version 	= "2.1",
	url 		= "http://hlmod.ru/forum/showthread.php?t=18977"
};

#pragma semicolon 1

//------------------------------------------------------------------------------
// DEFINES
//------------------------------------------------------------------------------
#define DEBUG 					0

#define TYPE_PANEL 				0
#define TYPE_MENU				1

#define MAXLENGTH_LINE 			128
#define MAXLENGTH_TITLE 		64
#define MAXLENGTH_TEXT 			64

#define SUCCESS					0
#define ERROR_UNKNOWN_TYPE		1
#define ERROR_NO_ACTIVE_HANDLE	2
#define ERROR_PANEL_OVERFLOWED	3
#define ERROR_MENU_NOT_FOUND	4

//------------------------------------------------------------------------------
// VARIABLES
//------------------------------------------------------------------------------
new Handle:	g_hTrieNameToHandle;
new Handle:	g_hTrieCommandToHandle;
new Handle:	g_hTrieBackHandleFromHandle;

new Handle:	g_hHandleType;
new Handle:	g_hPanelCommands;

new Handle:	g_hLastPanel[MAXPLAYERS+1];

new Handle:	g_hActiveHandle;

//------------------------------------------------------------------------------
// FORWARDS
//------------------------------------------------------------------------------
public OnPluginStart()
{
	g_hTrieNameToHandle = CreateTrie();
	g_hTrieCommandToHandle = CreateTrie();
	g_hTrieBackHandleFromHandle = CreateTrie();
	
	RegServerCmd("sm_mc_open",		Command_Open,				"sm_mc_open <client> <name>");
	RegServerCmd("sm_mc_openurl",	Command_OpenURL,			"sm_mc_openurl <client> <url>");

	g_hHandleType = CreateKeyValues("mc_types");
	g_hPanelCommands = CreateKeyValues("mc_panel_commands");
	
	decl String:sPath[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, sPath, sizeof(sPath) - 1, "configs/menu_creator.txt");
	new Handle:hFile = OpenFile(sPath, "r");
	
	if ( hFile == INVALID_HANDLE )
	{
		SetFailState("File '%s' not found or corrupted", sPath);
		return;
	}
	
	new iLineNumber = 1;
	decl String:sLine[MAXLENGTH_LINE];
	while ( !IsEndOfFile(hFile) && ReadFileLine(hFile, sLine, sizeof(sLine) - 1) )
	{
		TrimString(sLine);
		if ( StrContains(sLine, "\\") == 0 )
		{
			continue;
		}
		
		decl String:sBuffer[4][MAXLENGTH_LINE];
		new iArgCount = ExplodeString(sLine, "|", sBuffer, sizeof(sBuffer), sizeof(sBuffer[]));
		
		for ( new i = 0; i < iArgCount; i++ )
		{
			TrimString(sBuffer[i]);
		}
		ParseString(sLine, iLineNumber, sBuffer, iArgCount);
		iLineNumber++;
	}
	
	g_hActiveHandle = INVALID_HANDLE;
	CloseHandle(hFile);
}

//------------------------------------------------------------------------------
// SERVER COMMANDS
//------------------------------------------------------------------------------
ParseString(String:sSourceString[], num, String:sBuffer[][], argc)
{
	if ( argc < 1 || strlen(sBuffer[0]) == 0 )
	{
		return;
	}
	
	if ( StrEqual(sBuffer[0], "create", false) )
	{
		if ( argc < 3 )
		{
			LogError("Error at %d line (%s): syntax: create|name|type", num, sSourceString);
		}
		else
		{
			new iResult = CreateHandle(sBuffer[1], sBuffer[2]);
			if ( iResult == ERROR_UNKNOWN_TYPE )
			{
				LogError("Error at %d line (%s): unknown handle type '%s'", num, sSourceString, sBuffer[2]);
			}
		}
	}
	else if ( StrEqual(sBuffer[0], "regcmd", false) )
	{
		if ( argc < 2 )
		{
			LogError("Error at %d line (%s): syntax: regcmd|command|flag", num, sSourceString);
		}
		else
		{
			if ( argc == 3 && strlen(sBuffer[2]) == 0 )
			{
				argc = 2;
			}
			
			new iResult = RegisterCommand(sBuffer[1], sBuffer[2], argc);
			if ( iResult == ERROR_NO_ACTIVE_HANDLE )
			{
				LogError("Error at %d line (%s): active handle not found (you should use create first)", num, sSourceString);
			}
		}
	}
	else if ( StrEqual(sBuffer[0], "settitle", false) )
	{
		if ( argc < 2 )
		{
			LogError("Error at %d line (%s): syntax: settitle|title", num, sSourceString);
		}
		else
		{
			new iResult = SetTitle(sBuffer[1]);
			if ( iResult == ERROR_NO_ACTIVE_HANDLE )
			{
				LogError("Error at %d line (%s): active handle not found (you should use create first)", num, sSourceString);
			}
		}
	}
	else if ( StrEqual(sBuffer[0], "additem", false) )
	{
		if ( argc < 3 )
		{
			LogError("Error at %d line (%s): syntax: additem|text|command", num, sSourceString);
		}
		else
		{
			new iResult = AddItem(sBuffer[1], sBuffer[2]);
			if ( iResult == ERROR_NO_ACTIVE_HANDLE )
			{
				LogError("Error at %d line (%s): active handle not found (you should use create first)", num, sSourceString);
			}
			else if ( iResult == ERROR_PANEL_OVERFLOWED )
			{
				LogError("Error at %d line (%s): panel overflowed (more then 9 items)", num, sSourceString);
			}
		}
	}
	else if ( StrEqual(sBuffer[0], "addtext", false) )
	{
		if ( argc < 2 )
		{
			LogError("Error at %d line (%s): syntax: addtext|text", num, sSourceString);
		}
		else
		{
			new iResult = AddText(sBuffer[1]);
			if ( iResult == ERROR_NO_ACTIVE_HANDLE )
			{
				LogError("Error at %d line (%s): active handle not found (you should use create first)", num, sSourceString);
			}
		}
	}
	else if ( StrEqual(sBuffer[0], "setback", false) )
	{
		if ( argc < 1 )
		{
			LogError("Error at %d line (%s): syntax: setback|menu", num, sSourceString);
		}
		else
		{
			new iResult = SetBack(sBuffer[1]);
			if ( iResult == ERROR_NO_ACTIVE_HANDLE )
			{
				LogError("Error at %d line (%s): active handle not found (you should use create first)", num, sSourceString);
			}
			else if ( iResult == ERROR_MENU_NOT_FOUND )
			{
				LogError("Error at %d line (%s): back handle with name '%s' not found", num, sSourceString, sBuffer[1]);
			}
			else if ( iResult == ERROR_PANEL_OVERFLOWED )
			{
				LogError("Error at %d line (%s): panel overflowed (more then 9 items)", num, sSourceString);
			}
		}
	}
	else
	{
		LogError("Error at %d line (%s): unknown command %s", num, sSourceString, sBuffer[0]);
	}
}

CreateHandle(const String:sName[], const String:sType[])
{
	if ( StrEqual(sType, "menu") )
	{
		g_hActiveHandle = CreateMenu(Handle_MenuHandle);
		SetTrieValue(g_hTrieNameToHandle, sName, g_hActiveHandle);
		
		decl String:sHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", g_hActiveHandle);
		KvSetNum(g_hHandleType, sHandle, TYPE_MENU);
	}
	else if ( StrEqual(sType, "panel") )
	{
		g_hActiveHandle = CreatePanel();
		SetTrieValue(g_hTrieNameToHandle, sName, g_hActiveHandle);
		
		decl String:sHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", g_hActiveHandle);
		KvSetNum(g_hHandleType, sHandle, TYPE_PANEL);
	}
	else
	{
		return ERROR_UNKNOWN_TYPE;
	}
	
	return SUCCESS;
}

RegisterCommand(const String:sCommand[], const String:sAccessFlag[], argc)
{
	if ( g_hActiveHandle == INVALID_HANDLE )
	{
		return ERROR_NO_ACTIVE_HANDLE;
	}
	
	SetTrieValue(g_hTrieCommandToHandle, sCommand, g_hActiveHandle);
	if ( argc == 2 )
	{
		RegConsoleCmd(sCommand, Command_OpenHandle);
	}
	else
	{
		new iAccessFlag = ReadFlagString(sAccessFlag);
		RegAdminCmd(sCommand, Command_OpenHandle, iAccessFlag);
	}
	
	return SUCCESS;
}

SetTitle(const String:sTitle[])
{
	if ( g_hActiveHandle == INVALID_HANDLE )
	{
		return ERROR_NO_ACTIVE_HANDLE;
	}
	
	decl String:sHandle[16], String:_sTitle[MAXLENGTH_TITLE];
	Format(sHandle, sizeof(sHandle) - 1, "%d", g_hActiveHandle);
	Format(_sTitle, sizeof(_sTitle) - 1, "%s\n \n", sTitle);
	
	if ( KvGetNum(g_hHandleType, sHandle) == TYPE_PANEL )
	{
		SetPanelTitle(g_hActiveHandle, _sTitle);
	}
	else
	{
		SetMenuTitle(g_hActiveHandle, _sTitle);
	}
	
	return SUCCESS;
}

AddItem(const String:sText[], const String:sCommand[])
{
	if ( g_hActiveHandle == INVALID_HANDLE )
	{
		return ERROR_NO_ACTIVE_HANDLE;
	}
	
	decl String:sHandle[16], String:_sText[MAXLENGTH_TEXT];
	strcopy(_sText, sizeof(_sText) - 1, sText);
	Format(sHandle, sizeof(sHandle) - 1, "%d", g_hActiveHandle);
	
	ReplaceString(_sText, sizeof(_sText) - 1, "{newline}", "\n \n");
	ReplaceString(_sText, sizeof(_sText) - 1, "{slash}", "|");
		
	if ( KvGetNum(g_hHandleType, sHandle) == TYPE_PANEL )
	{
		new iPosition = DrawPanelItem(g_hActiveHandle, _sText);
		if ( iPosition != 0 )
		{
			KvRewind(g_hPanelCommands);
			if ( KvJumpToKey(g_hPanelCommands, sHandle, true) )
			{
				decl String:sPosition[4];
				IntToString(iPosition, sPosition, sizeof(sPosition) - 1);
				KvSetString(g_hPanelCommands, sPosition, sCommand);
			}
		}
		else
		{
			return ERROR_PANEL_OVERFLOWED;
		}
	}
	else
	{
		AddMenuItem(g_hActiveHandle, sCommand, _sText);
	}
	
	return SUCCESS;
}

AddText(const String:sText[])
{
	if ( g_hActiveHandle == INVALID_HANDLE )
	{
		return ERROR_NO_ACTIVE_HANDLE;
	}
	
	decl String:sHandle[16], String:_sText[MAXLENGTH_TEXT];
	strcopy(_sText, sizeof(_sText) - 1, sText);
	Format(sHandle, sizeof(sHandle) - 1, "%d", g_hActiveHandle);
	
	ReplaceString(_sText, sizeof(_sText) - 1, "{n}", "\n \n");
	ReplaceString(_sText, sizeof(_sText) - 1, "{vs}", "|");
	ReplaceString(_sText, sizeof(_sText) - 1, "{space}", " ");
		
	if ( KvGetNum(g_hHandleType, sHandle) == TYPE_PANEL )
	{
		DrawPanelItem(g_hActiveHandle, _sText, ITEMDRAW_RAWLINE);
	}
	else
	{
		AddMenuItem(g_hActiveHandle, NULL_STRING, _sText, ITEMDRAW_DISABLED);
	}
	
	return SUCCESS;
}

SetBack(const String:sBackName[])
{
	if ( g_hActiveHandle == INVALID_HANDLE )
	{
		return ERROR_NO_ACTIVE_HANDLE;
	}
	
	decl Handle:hBackHandle;
	if ( GetTrieValue(g_hTrieNameToHandle, sBackName, hBackHandle) )
	{
		decl String:sHandle[16], String:sBackHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", g_hActiveHandle);
		Format(sBackHandle, sizeof(sBackHandle) - 1, "%d", hBackHandle);
		SetTrieValue(g_hTrieBackHandleFromHandle, sHandle, hBackHandle);
		
		if ( KvGetNum(g_hHandleType, sHandle) == TYPE_PANEL )
		{
			new iPosition = DrawPanelItem(g_hActiveHandle, "Назад");
			if ( iPosition != 0 )
			{
				KvRewind(g_hPanelCommands);
				if ( KvJumpToKey(g_hPanelCommands, sHandle, true) )
				{
					decl String:sPosition[4], String:sCommand[128];
					IntToString(iPosition, sPosition, sizeof(sPosition) - 1);
					Format(sCommand, sizeof(sCommand) - 1, "sm_mc_open {client} %s", sBackName);
					
					KvSetString(g_hPanelCommands, sPosition, sCommand);
				}
			}
			else
			{
				return ERROR_PANEL_OVERFLOWED;
			}
		}
		else
		{
			SetMenuExitBackButton(g_hActiveHandle, true);
		}
	}
	else
	{
		return ERROR_MENU_NOT_FOUND;
	}
	
	return SUCCESS;
}

public Action:Command_Open(argc)
{
	if ( GetCmdArgs() != 2 )
	{
		PrintToServer("syntax: sm_mc_open <client> <name>");
		return Plugin_Handled;
	}
	
	decl String:sName[64], String:sClient[4];
	GetCmdArg(1, sClient, sizeof(sClient) - 1);
	GetCmdArg(2, sName, sizeof(sName) - 1);
	
	decl Handle:hHandle;
	if ( GetTrieValue(g_hTrieNameToHandle, sName, hHandle) )
	{
		new iTarget = StringToInt(sClient);
		decl String:sHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", hHandle);
		if ( KvGetNum(g_hHandleType, sHandle) == TYPE_PANEL )
		{
			g_hLastPanel[iTarget] = hHandle;
			SendPanelToClient(hHandle, iTarget, Handle_PanelHandle, MENU_TIME_FOREVER);
		}
		else
		{
			DisplayMenu(hHandle, iTarget, MENU_TIME_FOREVER);
		}
	}
	else
	{
		LogError("trying to open '%s' invalid handle", sName);
	}
	
	return Plugin_Handled;
}
public Action:Command_OpenURL(argc)
{
	if ( GetCmdArgs() != 2 )
	{
		PrintToServer("syntax: sm_mc_openurl <client> <url>");
		return Plugin_Handled;
	}
	
	decl String:sURL[128], String:sClient[4];
	GetCmdArg(1, sClient, sizeof(sClient) - 1);
	GetCmdArg(2, sURL, sizeof(sURL) - 1);
	
	new iTarget = StringToInt(sClient);
	ShowMOTDPanel(iTarget, " ", sURL, MOTDPANEL_TYPE_URL);
	
	return Plugin_Handled;
}

//------------------------------------------------------------------------------
// MENU HANDLE
//------------------------------------------------------------------------------
public Handle_MenuHandle(Handle:hMenu, MenuAction:action, iClient, iSlot)
{
	if ( action == MenuAction_Select )
	{
		decl String:sInfo[128];
		GetMenuItem(hMenu, iSlot, sInfo, sizeof(sInfo) - 1);
		
		if ( StrContains(sInfo, "{client}") != -1 )
		{
			decl String:sClient[8];
			IntToString(iClient, sClient, sizeof(sClient) - 1);
			ReplaceString(sInfo, sizeof(sInfo) - 1, "{client}", sClient);
		}
		if ( StrContains(sInfo, "{userid}") != -1 )
		{
			new iUserId = GetClientUserId(iClient);
			decl String:sUserId[8];
			IntToString(iUserId, sUserId, sizeof(sUserId) - 1);
			ReplaceString(sInfo, sizeof(sInfo) - 1, "{userid}", sUserId);
		}
		if ( StrContains(sInfo, "{name}") != -1 )
		{
			decl String:sName[MAX_NAME_LENGTH];
			GetClientName(iClient, sName, sizeof(sName) - 1);
			ReplaceString(sInfo, sizeof(sInfo) - 1, "{name}", sName);
		}
		
		#if DEBUG == 1
		PrintToServer("%s", sInfo);
		#endif
		ServerCommand("%s", sInfo);
	}
	else if ( action == MenuAction_Cancel && iSlot == MenuCancel_ExitBack )
	{
		decl String:sHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", hMenu);
		
		decl Handle:hBackHandle;
		if ( GetTrieValue(g_hTrieBackHandleFromHandle, sHandle, hBackHandle) )
		{
			decl String:sBackHandle[16];
			Format(sBackHandle, sizeof(sBackHandle) - 1, "%d", hBackHandle);
			if ( KvGetNum(g_hHandleType, sBackHandle) == TYPE_PANEL )
			{
				g_hLastPanel[iClient] = hBackHandle;
				SendPanelToClient(hBackHandle, iClient, Handle_PanelHandle, MENU_TIME_FOREVER);
			}
			else
			{
				DisplayMenu(hBackHandle, iClient, MENU_TIME_FOREVER);
			}
		}
		else
		{
			LogError("trying to open '%s' invalid handle", sHandle);
		}
	}
	
	return;
}

//------------------------------------------------------------------------------
// PANEL HANDLE
//------------------------------------------------------------------------------
public Handle_PanelHandle(Handle:hPanel, MenuAction:action, iClient, iSlot)
{
	if ( action == MenuAction_Select )
	{
		decl String:sHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", g_hLastPanel[iClient]);
		
		KvRewind(g_hPanelCommands);
		if ( KvJumpToKey(g_hPanelCommands, sHandle) )
		{
			decl String:sPosition[4], String:sCommand[128];
			IntToString(iSlot, sPosition, sizeof(sPosition) - 1);
			KvGetString(g_hPanelCommands, sPosition, sCommand, sizeof(sCommand) - 1);
			
			if ( StrContains(sCommand, "{client}") != -1 )
			{
				decl String:sClient[8];
				IntToString(iClient, sClient, sizeof(sClient) - 1);
				ReplaceString(sCommand, sizeof(sCommand) - 1, "{client}", sClient);
			}
			if ( StrContains(sCommand, "{userid}") != -1 )
			{
				new iUserId = GetClientUserId(iClient);
				decl String:sUserId[8];
				IntToString(iUserId, sUserId, sizeof(sUserId) - 1);
				ReplaceString(sCommand, sizeof(sCommand) - 1, "{userid}", sUserId);
			}
			if ( StrContains(sCommand, "{name}") != -1 )
			{
				decl String:sName[MAX_NAME_LENGTH];
				GetClientName(iClient, sName, sizeof(sName) - 1);
				ReplaceString(sCommand, sizeof(sCommand) - 1, "{name}", sName);
			}
			
			#if DEBUG == 1
			PrintToServer("%s", sCommand);
			#endif
			ServerCommand("%s", sCommand);
		}
		else
		{	
			PrintToServer("error: jump to %s failed", sHandle);
		}
	}
}

//------------------------------------------------------------------------------
// CLIENT-SIDE COMMANDS
//------------------------------------------------------------------------------
public Action:Command_OpenHandle(iClient, argc)
{
	if ( iClient == 0 )
	{
		PrintToServer("client-side only");
		return Plugin_Handled;
	}
	
	decl String:sCommand[32];
	GetCmdArg(0, sCommand, sizeof(sCommand) - 1);
	
	decl Handle:hHandle;
	if ( GetTrieValue(g_hTrieCommandToHandle, sCommand, hHandle) )
	{
		decl String:sHandle[16];
		Format(sHandle, sizeof(sHandle) - 1, "%d", hHandle);
		if ( KvGetNum(g_hHandleType, sHandle) == TYPE_PANEL )
		{
			g_hLastPanel[iClient] = hHandle;
			SendPanelToClient(hHandle, iClient, Handle_PanelHandle, MENU_TIME_FOREVER);
		}
		else
		{
			DisplayMenu(hHandle, iClient, MENU_TIME_FOREVER);
		}
	}
	else
	{
		LogError("command '%s' without a handle", sCommand);
	}
	
	return Plugin_Handled;
}